;;; BGPL: BEM - GNUPLOT plot menu list 
;;;
;;; 

;; A list of plots is diplayed which have been prepared by the BEM PLOT
;; command. The list serves as a menu from which functions can be applied
;; to the plot which is indicated by cursor position.  The list is shown
;; in an emacs major mode, bgpl-mode. Normal editing is inhibited. The
;; functions which can be invoked from the list are: preview, printed
;; output and plot deletion.

;; The plots are assumed to be present in a directory which is pointed to
;; by the shell variable "outputs". Each plot consists of a
;; Gnuplot-program named "bem_gpl.nnn.gpl", where "nnn" is a code to
;; discriminate different plots, and an associated data file
;; "bem_gpl.nnn.dat".

;;========================================================================
;; customization:
;; the following setq controls the number and 1-character-codes of
;; printers  for
;; postscript output. The actual output is done by a shell-script 
;; which is the contents of shell script "$bemprog/bem_gpl_psout.s". It must
;; take as 1st argument the 1-character-code for the printer and as
;; 2nd the name of the postscript file to be printed.
;; 
;; (at present in Goettingen: Agfa, HPIIIsi and file-output)

(setq bgpl-prt-list '("p" "h" "f")) 

;;=========================================================================

;;(get-buffer-create "test")

;; The following functions make the the menu list
;;
(defun bgpl-make () "make menu list of BEM-GNUPLOT plots"
  (interactive)
  (setq bgpl-buf "BEM-GP_plots_menu")
  (get-buffer-create "*sys-serv*")
  (get-buffer-create bgpl-buf)
  (switch-to-buffer bgpl-buf)
  (setq buffer-read-only nil)
  (setq bgpl-tmp (substitute-in-file-name "$Tempo"))
  (setq bgpl-dat (substitute-in-file-name "$bemout/plots"))

;; get a list of all plots
  (erase-buffer)
  (call-process "ls" nil bgpl-buf nil bgpl-dat)
  (goto-char 1)
  (while (not (eobp))
    (end-of-line)
    (setq eol (point))
    (beginning-of-line)
    (setq bol (point))
    (setq bgpl-lin (bgpl-get-line))
    (cond ((string-match "^.*\.\\(...\\)\.gpl" bgpl-lin)
	   (setq bgpl-num
		 (substring bgpl-lin (match-beginning 1) (match-end 1)))
	   (beginning-of-line)
	   (insert bgpl-num (make-string 60 ?\ ) "@")
	   (forward-line))
	  (t
	   (beginning-of-line)
	   (kill-line 1))))

;; reverse list (last one up) and set heading

  (sort-lines t 1 (point))
  (goto-char 1)
  (insert " #      model-id            name     x   y1   y2   ... " ?\n)
  (insert (make-string 100 ?\-) ?\n)
  (end-of-line)
  (setq bgpl-mrk1 (point)) ; mark end of headings
  (setq bgpl-col0 20) ; column-position of print-command

;; get the headers of individual plotfiles
  (get-buffer-create "gplprogs")
  (while (not (eobp))
    (setq bgpl-lin (bgpl-get-line))
    (setq bgpl-fil (substring  bgpl-lin (1+ (string-match "@.*" bgpl-lin))))
    (setq bgpl-fil1 (concat bgpl-dat "/" bgpl-fil))
    (set-buffer "gplprogs")
    (erase-buffer)
    (insert-file-contents bgpl-fil1)
    (goto-char 1)
    (setq bgpl-str (bgpl-get-line))
    (set-buffer bgpl-buf)
    (beginning-of-line)
    (forward-char 3);; 15.1.94
    (insert (substring bgpl-str 15) )
    (beginning-of-line)
    (forward-char 19)
    (insert ">     < ")
    (forward-line))

;; go to top of command area
  (goto-char 1)
  (forward-line 2)
  (forward-char bgpl-col0)
  (setq bgpl-pos0 (point))  
  (bgpl-mode))

(defun bgpl-refresh () ; refresh list (g-key)
  (interactive)
  (bgpl-make)
  (message "Menu list refreshed."))


;; The following functions allow cursor movements (up-arrow, and
;; down-arrow) and horizontal scrolling (left- and right-arrow)
;;
(defun bgpl-up ()
  (interactive)
  (forward-line -1)
  (if (< (point) bgpl-mrk1)
      (goto-char bgpl-pos0)
    (forward-char bgpl-col0)))

(defun bgpl-down ()
  (interactive)
  (forward-line 1)
  (if (eobp)
      (goto-char bgpl-pos0)
    (forward-char bgpl-col0)))

(defun bgpl-right ()
  (interactive)
  (scroll-left 50))

(defun bgpl-left ()
  (interactive)
  (scroll-right 50))


;; The following functions allow the insertion of command-marks which
;; will be executed by the x-key
;;
(defun bgpl-set-prt-key ()
 "In BGPL-mode: mark in command area key for printer for postscript output."
  (interactive)
  (beginning-of-line)
  (forward-char bgpl-col0)
  (setq buffer-read-only nil)
  (self-insert-command 1)
  (delete-char 1)
  (setq buffer-read-only t))

(defun bgpl-set-d ()
  "In BGPL-mode: mark plot for deletion"
  (interactive)
  (beginning-of-line)
  (forward-char bgpl-col0)
  (forward-char)
  (setq buffer-read-only nil)
  (insert "D")
  (delete-char 1)
  (setq buffer-read-only t))

(defun bgpl-unset () "In BGPL-mode: delete commands in command area"
  (interactive)
  (beginning-of-line)
  (forward-char bgpl-col0)
  (setq buffer-read-only nil)
  (insert "  ")
  (delete-char 2)
  (forward-char -2)
  (setq buffer-read-only t)
  (bgpl-down))

(defun bgpl-ditto () "ditto menu command in following line"
  (interactive)
  (beginning-of-line)
  (forward-char bgpl-col0)
  (let ((cmd ""))
    (setq cmd (buffer-substring (point) (+ (point) 2)))
    (forward-line)
    (cond
     ((eobp)
      (forward-line -1)
      (forward-char bgpl-col0))
     (t
      (forward-char bgpl-col0)
      (setq buffer-read-only nil)
      (insert cmd)
    (delete-char 2)
    (forward-char -2)
    (setq buffer-read-only t)))))

;; help-function: display valid printer commands in echo area
;;
(defun bgpl-show-prt ()
  (interactive)
  (princ "ps output commands: " t)
  (princ bgpl-prt-list t))

;; two simple utilities
;;
(defun bgpl-get-line () ; line from buffer into a string
  (let ((bol 0)(eol 0))
    (beginning-of-line)
    (setq bol (point))
    (end-of-line)
    (setq eol (point))
    (buffer-substring bol eol)))

(defun bgpl-get-time ()  ; system time as string for subdirectory name
(save-excursion 
  (get-buffer-create "time")
  (set-buffer "time")
  (erase-buffer)
  (call-process "date" nil "time" nil "+%H%M%S")
  (goto-char 1)
  (bgpl-get-line)))


;; The next function introduces "hard links" to the files to be
;; plotted in temporary directories. This is to prevent preemptive
;; deletion of the same files by other processes

(defun bgpl-link-to-tmp (nam1 nam2)
;; use a list-file which indicates the data files pertaining to this plot
  (save-excursion 
    (get-buffer-create "bgpl-dat-list")
    (set-buffer "bgpl-dat-list")
    (erase-buffer)
    (setq bgpl-fil (substitute-in-file-name
		    (concat bgpl-dat "/" nam1 ".lis")))
    (insert-file-contents bgpl-fil)
    (goto-char 1)
    (while (not (eobp))
      (setq bgpl-fil (bgpl-get-line))
      (setq bgpl-fil1 (substitute-in-file-name
		       (concat bgpl-dat "/" bgpl-fil)))
      (setq bgpl-fil2 (substitute-in-file-name
		       (concat bgpl-tmp "/" bgpl-fil)))
      (call-process "rm" nil "*sys-serv*" nil "-f" bgpl-fil2)
      (call-process "ln" nil "*sys-serv*" nil "-s" bgpl-fil1 bgpl-fil2)
      (forward-line)))
  (setq bgpl-fil1 (substitute-in-file-name
		   (concat bgpl-dat "/" nam1 ".gpl")))
  (setq bgpl-fil2 (substitute-in-file-name
		   (concat bgpl-tmp "/" nam2 ".gpl")))
  (call-process "rm" nil "*sys-serv*" nil "-f" bgpl-fil2)
  (call-process "ln" nil "*sys-serv*" nil "-s" bgpl-fil1 bgpl-fil2))


;; This function produces a preview using Gnuplot

(defun bgpl-preview () "In BGPL-mode: show preview of plot in current line."
  (interactive)
  (save-excursion
    (setq bgpl-lin (bgpl-get-line))
    (setq bgpl-num (substring bgpl-lin 0 3))
    (string-match "^.*@\\(.*\\).gpl.*$" bgpl-lin)
    (setq bgpl-nam (substring  bgpl-lin (match-beginning 1) (match-end 1)))
    (bgpl-link-to-tmp bgpl-nam "bem_gpl_prev")
    (cond
     ((equal (process-status "preview") 'run)
      (process-send-string "preview" "\n")) ; send RET to Gnuplot
     ((equal (process-status "preview") nil)
      (get-buffer-create "prev.gpl")
      (set-buffer "prev.gpl")
      (erase-buffer)
      (insert "cd \"" bgpl-tmp "\"\n")
      (let ((lop 0))
	(while (< lop 100)
	  (insert "load \"bem_gpl_prev.gpl\"\n") 
	  (insert "pause -1\n")
	  (setq lop (1+ lop))))
      (setq bgpl-fil1 (substitute-in-file-name
		       (concat bgpl-tmp "/prev.gpl")))
      (write-region 1 (point) bgpl-fil1 nil 1)
      (get-buffer-create "*preview*")
      (start-process "preview" "*preview*" "gnuplot" bgpl-fil1)
      (message "Gnuplot for preview started as loop. (Can be killed by C-z.)")
      (process-kill-without-query (get-process "preview")))
     (t
      (switch-to-buffer "*preview*")
      (message "Error in preview subprocess ?")))))


;; This functions stops the preview process

(defun bgpl-kill-preview () "C-z key in BGPL"
  (interactive)
  (kill-process "preview"))


;; The next two functions do the printing and deletion of plots as
;; demanded by the commands present in the command area in the buffer
;; (on screen).

(defun bgpl-execute () "do postscript output and file deletion (x-key)"
  (interactive)
  (setq bgpl-tmp0 bgpl-tmp)
  (setq bgpl-tmp (concat bgpl-tmp0 "/" (bgpl-get-time)))
  (call-process "mkdir" nil "*sys-serv*" nil bgpl-tmp)
;; make a list of empty lists for output demands, one for each printer
  (setq bgpl-ps-out-list (make-list (length bgpl-prt-list) '()))
  (setq dlist '())
  (save-excursion
    (goto-char bgpl-pos0)
    (while (not (eobp))
      (setq bgpl-lin (bgpl-get-line))
      (string-match "^.*@\\(.*\\).gpl.*$" bgpl-lin)
      (setq bgpl-nam (substring  bgpl-lin (match-beginning 1) (match-end 1)))
      (beginning-of-line)
      (forward-char bgpl-col0)
      (setq chr (buffer-substring (point) (1+ (point))))
      (let ((lop 0)
	    (xlist '())
	    (nlist '()))
	(while (< lop (length bgpl-prt-list))
	  (setq xlist (elt bgpl-ps-out-list lop))
	  (if (equal chr (elt bgpl-prt-list lop))
	      (setq xlist (cons bgpl-nam xlist)))
	  (setq nlist (cons xlist nlist))  
	  (setq lop (1+ lop)))
	(setq bgpl-ps-out-list (reverse nlist)))
      (forward-char 1)
      (setq chr (buffer-substring (point) (1+ (point))))
      (if (equal chr "D")
	  (setq dlist (cons bgpl-nam dlist)))
      (forward-line)))

  ;; start writing shell-script
  (get-buffer-create "bgpl-exec.s")
  (set-buffer "bgpl-exec.s")
  (erase-buffer)
  (insert "# shell-script by BEM-GNUPLT\n")

  ;; make exec-code for specific printers
  (let ((lop 0))
    (while (< lop (length bgpl-prt-list))
      (bgpl-gpl-ps-exec (elt bgpl-ps-out-list lop) (elt bgpl-prt-list lop))
      (setq lop (1+ lop))))
  (insert "exit")
  (cond
   ((equal (process-status "gplexec") 'run)
    (message "exec still running, nothing done."))
   ((equal (process-status "gplexec") nil)
    (setq bgpl-fil3 (substitute-in-file-name
		     (concat bgpl-tmp "/bgpl-exec.s")))
    (write-region 1 (point) bgpl-fil3 nil 1)
    (call-process "chmod" nil "*sys-serv*" nil "+x" bgpl-fil3)
    (get-buffer-create "*bgpl-exec*")
    (start-process "bgpl-exec" "*bgpl-exec*" bgpl-fil3)
    (message  "subprocess \"bgpl-exec\* started."))
  (t
   (switch-to-buffer "*bgpl-exec*")
   (message "Error in exec subprocess ?")))
  (setq bgpl-tmp bgpl-tmp0)
  (let ((lop 0))
    (while (< lop (length dlist))
      (setq bgpl-nam (elt dlist lop))
      (get-buffer-create "bgpl-dat-list")
      (set-buffer "bgpl-dat-list")
      (erase-buffer)
      (setq bgpl-fil (substitute-in-file-name
		      (concat bgpl-dat "/" bgpl-nam ".lis")))
      (insert-file-contents bgpl-fil)
      (goto-char 1)
      (while (not (eobp))
	(setq bgpl-fil (bgpl-get-line))
	(setq bgpl-fil1 (substitute-in-file-name
			 (concat bgpl-dat "/" bgpl-fil)))
(print bgpl-fil1 (get-buffer "test")) ;;!!!!!!!!!!!!!!!!!!!
	(delete-file bgpl-fil1)
	(forward-line))
      (setq fil1 (concat bgpl-dat "/" bgpl-nam ".gpl"))
      (setq fil2 (concat bgpl-dat "/" bgpl-nam ".lis"))
      (delete-file fil1)
      (delete-file fil2)
      (setq lop (1+ lop))))
  (message "Exec started, deletions done.")
  (bgpl-make))

(defun bgpl-gpl-ps-exec (xlist printer); write exec-lines for 1 printer
(cond (xlist
       (insert "echo entering gnuplot at: `date`\n")
       (insert "gnuplot<<@eof\nset terminal postscript\n")
       (insert (concat "cd \"" bgpl-tmp "\"\n"))
       (setq bgpl-fil0 (concat "bem_gpl." printer ".ps"))
       (insert (concat "set output \"" bgpl-fil0 "\"\n"))
       (let ((lop 0))
	 (while (< lop (length xlist))
	   (setq bgpl-nam (elt xlist lop))
	   (bgpl-link-to-tmp bgpl-nam  bgpl-nam)
	   (insert (concat  "load \"" bgpl-nam ".gpl\"\n"))
	   (setq lop (1+ lop))))
       (insert "@eof\n")
       (insert "rc=$?\necho rc=$rc\necho\n")
       (insert "\[ rc -eq 0 \] \|\| exit 11 \n")
       (insert
	"$bemprog/bem_gpl_psout.s " printer " " bgpl-tmp"/" bgpl-fil0 "\n")
       (insert "\[ rc -eq 0 \] \|\| exit 11 \n"))))



;; The following is the actual definition of the bgpl-mode

(defun bgpl-mode ()  "offers a menu list of plots which have been prepared
by BEM PLOT in Gnuplot - Format for viewing or printing or deletion.
Normal editing is inhibited. The following special keys are available:
a, h, f (or other, type \"?\"   for info)   =: mark for postscript output
d =: mark plot for deletion  ,         =  =:  copy command mark on next line
u =: undo command marks      ,         g  =:  refresh menu list
x =: execute marked commands ,         v  =:  present preview
? =: show valid ps-output commands
up-arrow, down-arrow     =:  move cursor in command area
right-arrow, left-arrow  =:  scroll screen horizontally"

  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'bgpl-mode)
  (setq mode-name "BGPL")
  (setq buffer-read-only t)
  (setq truncate-lines t)
  (use-local-map bgpl-mode-map))


;; The following expressions are "top-level", they are evaluated at
;; loading the present file
;;
;; Define key-usage in bgpl-mode

(defvar bgpl-mode-map nil "Local keymap for bgpl-mode buffers.")
(if bgpl-mode-map
    nil
  (setq bgpl-mode-map (make-keymap))
  (suppress-keymap bgpl-mode-map)
  (define-key bgpl-mode-map "v" 'bgpl-preview)
  (define-key bgpl-mode-map "g" 'bgpl-refresh) ; refresh menu list
  (define-key bgpl-mode-map "\C-z" 'bgpl-kill-preview)
  (define-key bgpl-mode-map " " 'bgpl-down)
  (define-key bgpl-mode-map "\t" 'bgpl-down)
  (define-key bgpl-mode-map "\r" 'bgpl-down)
  (define-key bgpl-mode-map "d" 'bgpl-set-d)
  (define-key bgpl-mode-map "u" 'bgpl-unset)
  (define-key bgpl-mode-map "=" 'bgpl-ditto)
  (define-key bgpl-mode-map "x" 'bgpl-execute)
  (define-key bgpl-mode-map "?" 'bgpl-show-prt)
  (define-key bgpl-mode-map "\M-OD" 'bgpl-left)
  (define-key bgpl-mode-map "\M-OC" 'bgpl-right)
  (define-key bgpl-mode-map "\M-OA" 'bgpl-up)
  (define-key bgpl-mode-map "\M-OB" 'bgpl-down))

(put 'bgpl-mode 'mode-class 'special)

;; Set up the definitions of 1-character-codes for postscript-printers

(let ((lop 0)) 
  (while (< lop (length bgpl-prt-list))
    (define-key bgpl-mode-map (elt bgpl-prt-list lop) 'bgpl-set-prt-key)
    (setq lop (1+ lop))))

;; enter BGPL-mode


(bgpl-mode)
(bgpl-make)

;; end of file bgpl




